#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
stacksearch is a library that provides a easy way to search on all stackexchange sites.

What it does is it sends a request to the stackexchange website, does some fancy HTML
parsing, then returns a python dict containing questions and answers.


"""

__version__ = "1.2.4.1"

__author__ = ["Bryan Hu"]
